#include <stdio.h>
#include <stdlib.h>	/* atoll, etc */
#include <string.h>	/* strerror, etc */
#if !defined(sun)
#include <stdint.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>	/* gettimeofday, etc */

void usage(FILE *fp, const char *me)
{
  fprintf(fp, "usage:\n\t%s [-s size] path\n", me);
}
#define timevalue(t) ((t).tv_sec + 1E-6*(t).tv_usec)
void status(FILE *fp, intmax_t size, intmax_t s,
	    struct timeval *start_tv, struct timeval *tv)
{
  double b = s/(timevalue(*tv) - timevalue(*start_tv))*8;
  char cs[] = "|/-\\";

  fprintf(fp, "\r%3jd%% %c %jdGB(%.1fGbps) %jdMB(%.1fMbps) %jdKB(%.1fKbps) ",
	  100*s/size, cs[tv->tv_sec%(sizeof(cs)/sizeof(cs[0])-1)],
	  s/(1024*1024*1024), b/(1024*1024*1024),
	  s/(1024*1024), b/(1024*1024),
	  s/1024, b/1024);
}
int main(int argc, char *argv[])
{
  int a;
  FILE *fp;
  intmax_t size = 1LL<<(32+2) /* 4G*2^2 */, i;
  char data[1024*1024*4+1];
  struct timeval start_tv, tv, previous_tv;

  for (a=1; a<argc; a++)
    if (!strcmp(argv[a], "--"))
      break;
    else if (!strcmp(argv[a], "-h")) {
      usage(stdout, argv[0]);
      return 0;
    }
    else if (!strcmp(argv[a], "-s"))
      size = atoll(argv[++a]);
    else
      break;

  memset(data, 0, sizeof(data));
  memset(data, ' ', sizeof(data)-1);
  if (!(a < argc)) {
    usage(stderr, argv[0]);
    return 1;
  }
  if (!(fp = fopen(argv[a], "w+"))) {
    fprintf(stderr, "error: cannot create '%s'.\n", argv[a]);
    return 1;
  }
  gettimeofday(&start_tv, NULL);
  for (i=0; i<size/((intmax_t)sizeof(data)-1); i++) {
    if (fprintf(fp, "%s", data) == -1) {
      fprintf(stderr, "error: %s\n", strerror(errno));
      return 1;
    }
    gettimeofday(&tv, NULL);
    if (!(tv.tv_sec > previous_tv.tv_sec)) continue;
    status(stdout, size, i*(sizeof(data)-1), &start_tv, &tv);
    fflush(stdout);
    previous_tv = tv;
  }
  memset(data, 0, sizeof(data));
  memset(data, ' ', size%(sizeof(data)-1));
  if (fprintf(fp, "%s", data) == -1) {
    fprintf(stderr, "error: %s\n", strerror(errno));
    return 1;
  }
  gettimeofday(&tv, NULL);
  status(stdout, size, i*(sizeof(data)-1), &start_tv, &tv);
  fprintf(stdout, "\n");
  return 0;
}
